@@ -7,14 +7,13 @@ import numpy as np
 import pandas as pd
 import types
 
+from .strips import strips
 from ..utils import cross_join, match
 from ..exceptions import PlotnineError, PlotnineWarning
 from ..scales.scales import Scales
 
 # For default matplotlib backend
 with suppress(ImportError):
-    import matplotlib.text as mtext
-    import matplotlib.patches as mpatch
     from matplotlib.ticker import locale, FixedFormatter
 
 
@@ -81,6 +80,8 @@ class facet:
     num_vars_y = 0
     # ggplot object that the facet belongs to
     plot = None
+    # Facet strips
+    strips = None
 
     def __init__(self, scales='fixed', shrink=True,
                  labeller='label_value', as_table=True,
@@ -236,6 +237,26 @@ class facet:
 
         return self
 
+    def make_ax_strips(self, layout_info, ax):
+        """
+        Create strips for the facet
+
+        Parameters
+        ----------
+        layout_info : dict-like
+            Layout information. Row from the layout table
+
+        ax : axes
+            Axes to label
+        """
+        return []
+
+    def initialise_strips(self):
+        """
+        Initialise strips for the facet
+        """
+        self.strips = strips.initialise(self)
+
     def set_limits_breaks_and_labels(self, panel_params, ax):
         # limits
         ax.set_xlim(panel_params.x.range)
@@ -353,186 +374,6 @@ class facet:
         """
         pass
 
-    def inner_strip_margins(self, location):
-        if location == 'right':
-            strip_name = 'strip_text_y'
-            side1, side2 = 'l', 'r'
-        else:
-            strip_name = 'strip_text_x'
-            side1, side2 = 't', 'b'
-
-        try:
-            margin = self.theme.themeables.property(
-                strip_name, 'margin')
-        except KeyError:
-            m1, m2 = 3, 3
-        else:
-            m1 = margin.get_as(side1, 'pt')
-            m2 = margin.get_as(side2, 'pt')
-
-        return m1, m2
-
-    def strip_size(self, location='top', num_lines=None):
-        """
-        Breadth of the strip background in inches
-
-        Parameters
-        ----------
-        location : str in ``['top', 'right']``
-            Location of the strip text
-        num_lines : int
-            Number of text lines
-        """
-        dpi = 72
-        theme = self.theme
-        get_property = theme.themeables.property
-
-        if location == 'right':
-            strip_name = 'strip_text_y'
-            num_lines = num_lines or self.num_vars_y
-        else:
-            strip_name = 'strip_text_x'
-            num_lines = num_lines or self.num_vars_x
-
-        if not num_lines:
-            return 0
-
-        # The facet labels are placed onto the figure using
-        # transAxes dimensions. The line height and line
-        # width are mapped to the same [0, 1] range
-        # i.e (pts) * (inches / pts) * (1 / inches)
-        try:
-            fontsize = get_property(strip_name, 'size')
-        except KeyError:
-            fontsize = float(theme.rcParams.get('font.size', 10))
-
-        try:
-            linespacing = get_property(strip_name, 'linespacing')
-        except KeyError:
-            linespacing = 1
-
-        # margins on either side of the strip text
-        m1, m2 = self.inner_strip_margins(location)
-        # Using figure.dpi value here does not workout well!
-        breadth = (linespacing*fontsize) * num_lines / dpi
-        breadth = breadth + (m1 + m2) / dpi
-        return breadth
-
-    def strip_dimensions(self, text_lines, location, ax):
-        """
-        Calculate the dimension
-
-        Returns
-        -------
-        out : types.SimpleNamespace
-            A structure with all the coordinates required
-            to draw the strip text and the background box.
-        """
-        dpi = 72
-        num_lines = len(text_lines)
-        get_property = self.theme.themeables.property
-        bbox = ax.get_window_extent().transformed(
-            self.figure.dpi_scale_trans.inverted())
-        ax_width, ax_height = bbox.width, bbox.height  # in inches
-        strip_size = self.strip_size(location, num_lines)
-        m1, m2 = self.inner_strip_margins(location)
-        m1, m2 = m1/dpi, m2/dpi
-        margin = 0  # default
-
-        if location == 'right':
-            box_x = 1
-            box_y = 0
-            box_width = strip_size/ax_width
-            box_height = 1
-            # y & height properties of the background slide and
-            # shrink the strip vertically. The y margin slides
-            # it horizontally.
-            with suppress(KeyError):
-                box_y = get_property('strip_background_y', 'y')
-            with suppress(KeyError):
-                box_height = get_property('strip_background_y', 'height')
-            with suppress(KeyError):
-                margin = get_property('strip_margin_y')
-            x = 1 + (strip_size-m2+m1) / (2*ax_width)
-            y = (2*box_y+box_height)/2
-            # margin adjustment
-            hslide = 1 + margin*strip_size/ax_width
-            x *= hslide
-            box_x *= hslide
-        else:
-            box_x = 0
-            box_y = 1
-            box_width = 1
-            box_height = strip_size/ax_height
-            # x & width properties of the background slide and
-            # shrink the strip horizontally. The y margin slides
-            # it vertically.
-            with suppress(KeyError):
-                box_x = get_property('strip_background_x', 'x')
-            with suppress(KeyError):
-                box_width = get_property('strip_background_x', 'width')
-            with suppress(KeyError):
-                margin = get_property('strip_margin_x')
-            x = (2*box_x+box_width)/2
-            y = 1 + (strip_size-m1+m2)/(2*ax_height)
-            # margin adjustment
-            vslide = 1 + margin*strip_size/ax_height
-            y *= vslide
-            box_y *= vslide
-
-        dimensions = types.SimpleNamespace(
-            x=x, y=y, box_x=box_x, box_y=box_y,
-            box_width=box_width,
-            box_height=box_height)
-        return dimensions
-
-    def draw_strip_text(self, text_lines, location, ax):
-        """
-        Create a background patch and put a label on it
-        """
-        themeable = self.figure._themeable
-        dim = self.strip_dimensions(text_lines, location, ax)
-
-        if location == 'right':
-            rotation = -90
-            label = '\n'.join(reversed(text_lines))
-        else:
-            rotation = 0
-            label = '\n'.join(text_lines)
-
-        rect = mpatch.FancyBboxPatch((dim.box_x, dim.box_y),
-                                     width=dim.box_width,
-                                     height=dim.box_height,
-                                     facecolor='lightgrey',
-                                     edgecolor='None',
-                                     transform=ax.transAxes,
-                                     zorder=2.2,  # > ax line & boundary
-                                     boxstyle='square, pad=0',
-                                     clip_on=False)
-
-        text = mtext.Text(dim.x, dim.y, label,
-                          rotation=rotation,
-                          verticalalignment='center',
-                          horizontalalignment='center',
-                          transform=ax.transAxes,
-                          zorder=3.3,  # > rect
-                          clip_on=False)
-
-        ax.add_artist(rect)
-        ax.add_artist(text)
-
-        for key in ('strip_text_x', 'strip_text_y',
-                    'strip_background_x', 'strip_background_y'):
-            if key not in themeable:
-                themeable[key] = []
-
-        if location == 'right':
-            themeable['strip_background_y'].append(rect)
-            themeable['strip_text_y'].append(text)
-        else:
-            themeable['strip_background_x'].append(rect)
-            themeable['strip_text_x'].append(text)
-
     def check_axis_text_space(self):
         _adjust = self.theme.themeables.get('subplots_adjust')
         if _adjust:
@@ -595,8 +436,11 @@ def combine_vars(data, environment=None, vars=None, drop=True):
     for i, value in enumerate(values):
         if has_all[i] or len(value.columns) == 0:
             continue
-        old = base.loc[:, base.columns - value.columns]
-        new = value.loc[:, base.columns & value.columns].drop_duplicates()
+        old = base.loc[:, base.columns.difference(value.columns)]
+        new = value.loc[
+            :,
+            base.columns.intersection(value.columns)
+        ].drop_duplicates()
         if not drop:
             new = unique_combs(new)
         base = base.append(cross_join(old, new), ignore_index=True)
