@@ -3,23 +3,27 @@ import os
 import sys
 from copy import deepcopy
 from contextlib import suppress
+from itertools import chain
 from types import SimpleNamespace as NS
 from warnings import warn
 
 import pandas as pd
 import matplotlib as mpl
 import matplotlib.pyplot as plt
-from matplotlib.offsetbox import AnchoredOffsetbox
 import matplotlib.transforms as mtransforms
+from matplotlib.offsetbox import AnchoredOffsetbox
+from matplotlib.backends.backend_pdf import PdfPages
 from patsy.eval import EvalEnvironment
 
-from .aes import aes, make_labels
+from .mapping.aes import aes, make_labels
 from .layer import Layers
 from .facets import facet_null
 from .facets.layout import Layout
 from .options import get_option
 from .themes.theme import theme, theme_get
-from .utils import to_inches, from_inches, defaults, order_as_mapping_data
+from .utils import (
+        to_inches, from_inches, defaults, order_as_mapping_data, ungroup
+        )
 from .exceptions import PlotnineError, PlotnineWarning
 from .scales.scales import Scales
 from .coords import coord_cartesian
@@ -69,7 +73,7 @@ class ggplot:
         self.layers = Layers()
         self.guides = guides()
         self.scales = Scales()
-        self.theme = None
+        self.theme = theme_get()
         self.coordinates = coord_cartesian()
         self.environment = environment or EvalEnvironment.capture(1)
         self.layout = None
@@ -77,16 +81,20 @@ class ggplot:
         self.watermarks = []
         self.axs = None
 
+    def __str__(self):
+        """
+        Print/show the plot
+        """
+        self.draw(show=True)
+
+        # Return and empty string so that print(p) is "pretty"
+        return ''
+
     def __repr__(self):
         """
         Print/show the plot
         """
-        # Do not draw if drawn already.
-        # This prevents a needless error when reusing figure & axes
-        # in the jupyter notebook.
-        if not self.figure:
-            self.draw()
-        plt.show()
+        self.__str__()
         return '<ggplot: (%d)>' % self.__hash__()
 
     def __deepcopy__(self, memo):
@@ -118,6 +126,10 @@ class ggplot:
             return other.__radd__(self, inplace=True)
         except TypeError:
             return other.__radd__(self)
+        except AttributeError as err:
+            if other is None:
+                return self.__add__(other)
+            raise err
 
     def __add__(self, other):
         """
@@ -134,6 +146,8 @@ class ggplot:
             for item in other:
                 self += item
             return self
+        elif other is None:
+            return deepcopy(self)
         else:
             return other.__radd__(self)
 
@@ -141,6 +155,7 @@ class ggplot:
         """
         Overload the >> operator to receive a dataframe
         """
+        other = ungroup(other)
         if isinstance(other, pd.DataFrame):
             if self.data is None:
                 self.data = other
@@ -152,7 +167,7 @@ class ggplot:
             raise TypeError(msg.format(type(other)))
         return self
 
-    def draw(self, return_ggplot=False):
+    def draw(self, return_ggplot=False, show=False):
         """
         Render the complete plot
 
@@ -160,6 +175,8 @@ class ggplot:
         ----------
         return_ggplot : bool
             If ``True``, return ggplot object.
+        show : bool (default: False)
+            Whether to show the plot.
 
         Returns
         -------
@@ -174,48 +191,40 @@ class ggplot:
         This method does not modify the original ggplot object. You can
         get the modified ggplot object with :py:`return_ggplot=True`.
         """
-        # Pandas deprecated is_copy, and when we create new dataframes
-        # from slices we do not want complaints. We always uses the
-        # new frames knowing that they are separate from the original.
-        with pd.option_context('mode.chained_assignment', None):
-            return self._draw(return_ggplot)
+        # Do not draw if drawn already.
+        # This prevents a needless error when reusing
+        # figure & axes in the jupyter notebook.
+        if self.figure:
+            return (self.figure, self) if return_ggplot else self.figure
 
-    def _draw(self, return_ggplot=False):
         # Prevent against any modifications to the users
         # ggplot object. Do the copy here as we may/may not
         # assign a default theme
         self = deepcopy(self)
-        self._build()
+        with plot_context(self, show=show):
+            self._build()
 
-        # If no theme we use the default
-        self.theme = self.theme or theme_get()
+            # setup
+            figure, axs = self._create_figure()
+            self._setup_parameters()
+            self.facet.strips.generate()
+            self._resize_panels()
 
-        try:
-            with mpl.rc_context():
-                # setup & rcparams theming
-                self.theme.apply_rcparams()
-                figure, axs = self._create_figure()
-                self._setup_parameters()
-                self._resize_panels()
-                # Drawing
-                self._draw_layers()
-                self._draw_labels()
-                self._draw_breaks_and_labels()
-                self._draw_legend()
-                self._draw_title()
-                self._draw_watermarks()
-                # Artist object theming
-                self._apply_theme()  # !!
-        except Exception as err:
-            if self.figure is not None:
-                plt.close(self.figure)
-            raise err
+            # Drawing
+            self._draw_layers()
+            self._draw_labels()
+            self._draw_breaks_and_labels()
+            self._draw_legend()
+            self._draw_title()
+            self._draw_watermarks()
 
-        if return_ggplot:
-            output = self.figure, self
-        else:
-            output = self.figure
+            # Artist object theming
+            self._apply_theme()
 
+            if return_ggplot:
+                output = self.figure, self
+            else:
+                output = self.figure
         return output
 
     def _draw_using_figure(self, figure, axs):
@@ -234,24 +243,16 @@ class ggplot:
             Array of Axes onto which to draw the plots
         """
         self = deepcopy(self)
-        self._build()
-
-        self.theme = self.theme or theme_get()
         self.figure = figure
         self.axs = axs
-
-        try:
-            with mpl.rc_context():
-                self.theme.apply_rcparams()
-                self._setup_parameters()
-                self._draw_layers()
-                self._draw_breaks_and_labels()
-                self._draw_legend()
-                self._apply_theme()
-        except Exception as err:
-            if self.figure is not None:
-                plt.close(self.figure)
-            raise err
+        with plot_context(self):
+            self._build()
+            self._setup_parameters()
+            self.facet.strips.generate()
+            self._draw_layers()
+            self._draw_breaks_and_labels()
+            self._draw_legend()
+            self._apply_theme()
 
         return self
 
@@ -276,8 +277,8 @@ class ggplot:
         # Update the label information for the plot
         layers.update_labels(self)
 
-        # Give each layer a copy of the data that it will need
-        layers.generate_data(self.data)
+        # Give each layer a copy of the data and the mappings
+        layers.prepare(self)
 
         # Initialise panels, add extra data for margins & missing
         # facetting variables, and add on a PANEL variable to data
@@ -346,6 +347,7 @@ class ggplot:
             figure=self.figure,
             axs=self.axs
         )
+        self.facet.initialise_strips()
 
         # layout
         self.layout.axs = self.axs
@@ -392,15 +394,15 @@ class ggplot:
         """
         Draw breaks and labels
         """
-        # Decorate the axes
-        #   - xaxis & yaxis breaks, labels, limits, ...
-        #   - facet labels a.k.a strip text
+        # 1. Draw facet labels a.k.a strip text
+        # 2. Decorate the axes
+        #      - xaxis & yaxis breaks, labels, limits, ...
         #
         # pidx is the panel index (location left to right, top to bottom)
+        self.facet.strips.draw()
         for pidx, layout_info in self.layout.layout.iterrows():
             ax = self.axs[pidx]
             panel_params = self.layout.panel_params[pidx]
-            self.facet.draw_label(layout_info, ax)
             self.facet.set_limits_breaks_and_labels(panel_params, ax)
 
             # Remove unnecessary ticks and labels
@@ -444,8 +446,8 @@ class ggplot:
         with suppress(KeyError):
             strip_margin_y = get_property('strip_margin_y')
 
-        right_strip_width = self.facet.strip_size('right')
-        top_strip_height = self.facet.strip_size('top')
+        right_strip_width = self.facet.strips.breadth('right')
+        top_strip_height = self.facet.strips.breadth('top')
 
         # Other than when the legend is on the right the rest of
         # the computed x, y locations are not gauranteed not to
@@ -580,16 +582,19 @@ class ggplot:
             pad = margin.get_as('b', 'in')
 
         try:
-            strip_margin_x = get_property('strip_margin_x')
+            strip_margin_y = get_property('strip_margin_y')
         except KeyError:
-            strip_margin_x = 0
+            strip_margin_y = 0
 
-        line_size = fontsize / 72.27
+        dpi = 72.27
+        line_size = fontsize / dpi
         num_lines = len(title.split('\n'))
         title_size = line_size * linespacing * num_lines
-        strip_height = self.facet.strip_size('top')
+        strip_height = self.facet.strips.breadth('top')
+        # strip_height = 5.820833333333334
+        # print(strip_height)
         # vertical adjustment
-        strip_height *= (1 + strip_margin_x)
+        strip_height *= (1 + strip_margin_y)
 
         x = 0.5
         y = top + (strip_height+title_size/2+pad)/H
@@ -680,8 +685,6 @@ class ggplot:
                       'format': format}
         fig_kwargs.update(kwargs)
 
-        figure = [None]  # nonlocal
-
         # filename, depends on the object
         if filename is None:
             ext = format if format else 'pdf'
@@ -693,9 +696,6 @@ class ggplot:
         # Preserve the users object
         self = deepcopy(self)
 
-        # theme
-        self.theme = self.theme or theme_get()
-
         # The figure size should be known by the theme
         if width is not None and height is not None:
             width = to_inches(width, units)
@@ -715,41 +715,19 @@ class ggplot:
                 "not pixels). If you are sure you want these "
                 "dimensions, use 'limitsize=False'.".format(width, height))
 
-        if dpi is None:
-            try:
-                self.theme.themeables.property('dpi')
-            except KeyError:
-                self.theme = self.theme + theme(dpi=100)
-        else:
-            self.theme = self.theme + theme(dpi=dpi)
-
         if verbose:
             warn("Saving {0} x {1} {2} image.".format(
                  from_inches(width, units),
                  from_inches(height, units), units), PlotnineWarning)
             warn('Filename: {}'.format(filename), PlotnineWarning)
 
-        # Helper function so that we can clean up when it fails
-        def _save():
-            fig = figure[0] = self.draw()
-
-            # savefig ignores the figure face & edge colors
-            facecolor = fig.get_facecolor()
-            edgecolor = fig.get_edgecolor()
-            if facecolor:
-                fig_kwargs['facecolor'] = facecolor
-            if edgecolor:
-                fig_kwargs['edgecolor'] = edgecolor
+        if dpi is not None:
+            self.theme = self.theme + theme(dpi=dpi)
 
+        fig, p = self.draw(return_ggplot=True)
+        with plot_context(p):
             fig.savefig(filename, **fig_kwargs)
-
-        try:
-            _save()
-        except Exception as err:
-            figure[0] and plt.close(figure[0])
-            raise err
-        else:
-            figure[0] and plt.close(figure[0])
+            plt.close(fig)
 
 
 def ggsave(plot, *arg, **kwargs):
@@ -819,26 +797,20 @@ def save_as_pdf_pages(plots, filename=None, path=None, verbose=True, **kwargs):
     >>> plot.save('filename.pdf', height=6, width=8)
     >>> save_as_pdf_pages([plot + theme(figure_size=(8, 6))])
     """
-    from itertools import chain
-
-    from matplotlib.backends.backend_pdf import PdfPages
-
     # as in ggplot.save()
     fig_kwargs = {'bbox_inches': 'tight'}
     fig_kwargs.update(kwargs)
 
-    figure = [None]
-
-    # If plots is already an iterator, this is a no-op; otherwise convert a
-    # list, etc. to an iterator
+    # If plots is already an iterator, this is a no-op; otherwise
+    # convert a list, etc. to an iterator
     plots = iter(plots)
-    peek = []
 
     # filename, depends on the object
     if filename is None:
-        # Take the first element from the iterator, store it, and use it to
-        # generate a file name
+        # Take the first element from the iterator, store it, and
+        # use it to generate a file name
         peek = [next(plots)]
+        plots = chain(peek, plots)
         filename = peek[0]._save_filename('pdf')
 
     if path:
@@ -849,26 +821,52 @@ def save_as_pdf_pages(plots, filename=None, path=None, verbose=True, **kwargs):
 
     with PdfPages(filename) as pdf:
         # Re-add the first element to the iterator, if it was removed
-        for plot in chain(peek, plots):
-            try:
-                fig = figure[0] = plot.draw()
+        for plot in plots:
+            fig = plot.draw()
+            # Save as a page in the PDF file
+            pdf.savefig(fig, **fig_kwargs)
 
-                # as in ggplot.save()
-                facecolor = fig.get_facecolor()
-                edgecolor = fig.get_edgecolor()
-                if facecolor:
-                    fig_kwargs['facecolor'] = facecolor
-                if edgecolor:
-                    fig_kwargs['edgecolor'] = edgecolor
+            # To conserve memory when plotting a large number of pages,
+            # close the figure whether or not there was an exception
+            plt.close(fig)
 
-                # Save as a page in the PDF file
-                pdf.savefig(figure[0], **fig_kwargs)
-            except AttributeError as err:
-                msg = 'non-ggplot object of %s: %s' % (type(plot), plot)
-                raise TypeError(msg) from err
-            except Exception:
-                raise
-            finally:
-                # Close the figure whether or not there was an exception, to
-                # conserve memory when plotting a large number of pages
-                figure[0] and plt.close(figure[0])
+
+class plot_context:
+    """
+    Context to setup the environment within with the plot is built
+
+    Parameters
+    ----------
+    plot : ggplot
+        ggplot object to be built within the context.
+    show : bool (default: False)
+        Whether to show (``plt.show()``) the plot before the context
+        exits.
+    """
+    def __init__(self, plot, show=False):
+        self.plot = plot
+        self.show = show
+
+    def __enter__(self):
+        self.rc_context = mpl.rc_context(self.plot.theme.rcParams)
+        # Pandas deprecated is_copy, and when we create new dataframes
+        # from slices we do not want complaints. We always uses the
+        # new frames knowing that they are separate from the original.
+        self.pd_option_context = pd.option_context(
+            'mode.chained_assignment', None
+        )
+        self.rc_context.__enter__()
+        self.pd_option_context.__enter__()
+        return self
+
+    def __exit__(self, exc_type, exc_value, exc_traceback):
+        if exc_type is None:
+            if self.show:
+                plt.show()
+        else:
+            # There is an exception, close any figure
+            if self.plot.figure is not None:
+                plt.close(self.plot.figure)
+
+        self.rc_context.__exit__(exc_type, exc_value, exc_traceback)
+        self.pd_option_context.__exit__(exc_type, exc_value, exc_traceback)
