@@ -1,6 +1,5 @@
 from contextlib import suppress
 from copy import deepcopy, copy
-from collections import OrderedDict
 from functools import partial
 from types import SimpleNamespace as NS
 from warnings import warn
@@ -13,7 +12,7 @@ from mizani.breaks import date_breaks
 from mizani.formatters import date_format
 from mizani.transforms import gettrans
 
-from ..aes import is_position_aes, rename_aesthetics
+from ..mapping.aes import is_position_aes, rename_aesthetics
 from ..doctools import document
 from ..exceptions import PlotnineError, PlotnineWarning
 from ..utils import match, waiver, is_waive, Registry
@@ -437,7 +436,8 @@ class scale_discrete(scale):
                 except KeyError:
                     pal_match.append(self.na_value)
         else:
-            pal = np.asarray(pal)
+            if not isinstance(pal, np.ndarray):
+                pal = np.asarray(pal, dtype=object)
             idx = np.asarray(match(x, limits))
             try:
                 pal_match = pal[idx]
@@ -492,7 +492,7 @@ class scale_discrete(scale):
             in_domain = breaks
         pos = match(in_domain, breaks)
         tups = zip(in_domain, pos)
-        return OrderedDict(sorted(tups, key=lambda t: t[1]))
+        return dict(sorted(tups, key=lambda t: t[1]))
 
     def get_labels(self, breaks=None):
         """
@@ -894,15 +894,14 @@ class scale_continuous(scale):
         """
         Generate labels for the axis or legend
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         breaks: None or array-like
-            None ->result of self.breaks
-        mask: np.array(...,dtype=bool)
+            If None, use self.breaks.
+        mask: array[bool]
             Restrict returned labels to those with True
             in mask (used to exclude labels outside of a scale's
             domain even if the user defined them)
-
         """
         if breaks is None:
             breaks = self.get_breaks()
