@@ -3,7 +3,7 @@ import subprocess
 from pathlib import Path
 
 
-if not os.environ.get('TRAVIS'):
+if not os.environ.get('CI') and os.environ.get('TRAVIS'):
     def test_flake8():
         plotnine_dir = str(Path(__file__).parent.parent.absolute())
         p = subprocess.Popen(
